<?php

declare(strict_types=1);

namespace Dedi\SyliusVoucherPlugin\Operator;

use Dedi\SyliusVoucherPlugin\EmailManager\VoucherOrderEmailManagerInterface;
use Dedi\SyliusVoucherPlugin\Factory\VoucherFactoryInterface;
use Dedi\SyliusVoucherPlugin\Model\ProductInterface;
use Dedi\SyliusVoucherPlugin\Model\VoucherInterface;
use Dedi\SyliusVoucherPlugin\Repository\VoucherRepositoryInterface;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\EntityManagerInterface;
use Sylius\Component\Core\Model\OrderInterface;
use Sylius\Component\Core\Model\OrderItemInterface;
use Sylius\Component\Core\Model\OrderItemUnitInterface;
use Webmozart\Assert\Assert;

/**
 * This class' responsibility is to operate on vouchers bought on an order
 * It does NOT handle vouchers used to buy an order
 */
final class OrderVoucherOperator implements OrderVoucherOperatorInterface
{
    private readonly VoucherFactoryInterface $voucherFactory;

    private readonly VoucherRepositoryInterface $voucherRepository;

    private readonly EntityManagerInterface $voucherManager;

    private readonly VoucherOrderEmailManagerInterface $voucherOrderEmailManager;

    public function __construct(
        VoucherFactoryInterface $voucherFactory,
        VoucherRepositoryInterface $voucherRepository,
        EntityManagerInterface $voucherManager,
        VoucherOrderEmailManagerInterface $voucherOrderEmailManager,
    ) {
        $this->voucherFactory = $voucherFactory;
        $this->voucherRepository = $voucherRepository;
        $this->voucherManager = $voucherManager;
        $this->voucherOrderEmailManager = $voucherOrderEmailManager;
    }

    public function create(OrderInterface $order): void
    {
        $items = self::getOrderItemsThatAreVouchers($order);

        if (count($items) === 0) {
            return;
        }

        foreach ($items as $item) {
            /** @var OrderItemUnitInterface $unit */
            foreach ($item->getUnits() as $unit) {
                $voucher = $this->voucherFactory->createFromOrderItemUnit($unit);

                $this->voucherManager->persist($voucher);
            }
        }

        $this->voucherManager->flush();
    }

    public function enable(OrderInterface $order): void
    {
        $vouchers = $this->getVouchers($order);

        if (count($vouchers) === 0) {
            return;
        }

        foreach ($vouchers as $voucher) {
            $voucher->enable();
        }

        $this->voucherManager->flush();
    }

    /**
     * Calls when Order this VoucherCode was bought at
     * become cancelled
     */
    public function disable(OrderInterface $order): void
    {
        $vouchers = $this->getVouchers($order);

        if (count($vouchers) === 0) {
            return;
        }

        foreach ($vouchers as $voucher) {
            $voucher->disable();
        }

        $this->voucherManager->flush();
    }

    public function send(OrderInterface $order): void
    {
        $vouchers = $this->getVouchers($order);

        if (count($vouchers) === 0) {
            return;
        }

        $this->voucherOrderEmailManager->sendEmailWithVouchers($order, $vouchers);
    }

    /**
     * Returns all the vouchers that were bought on the given order
     *
     * @return VoucherInterface[]
     */
    private function getVouchers(OrderInterface $order): array
    {
        $vouchers = [];

        $items = self::getOrderItemsThatAreVouchers($order);
        foreach ($items as $item) {
            /** @var OrderItemUnitInterface $unit */
            foreach ($item->getUnits() as $unit) {
                $voucher = $this->voucherRepository->findOneByOrderItemUnit($unit);
                if (null === $voucher) {
                    continue;
                }

                $vouchers[] = $voucher;
            }
        }

        return $vouchers;
    }

    /**
     * @return Collection|OrderItemInterface[]
     */
    private static function getOrderItemsThatAreVouchers(OrderInterface $order): Collection
    {
        return $order->getItems()->filter(static function (OrderItemInterface $item): bool {
            /** @var ProductInterface|null $product */
            $product = $item->getProduct();

            Assert::isInstanceOf($product, ProductInterface::class);

            return $product->isVoucher();
        });
    }
}
